@@ -4,24 +4,13 @@
 #' @importFrom dplyr n_distinct
 #' @importFrom stats na.omit
 #' @keywords internal
-check_args <- function(mc, dotArgs) {
+check_args <- function(mc, dotArgs, caller) {
   
-  caller      <- sub(".+::","",as.character(sys.call(-1))[1])
   pf          <- parent.frame()
   errmsg      <- character()
-  caller_orig <- caller
   
-  if (caller == "FUN") {
+  if (mc[[1]] == "FUN" || mc$x == "dd[x, , drop = FALSE]") {
     pf$flag_by <- TRUE
-    # When stby() was called, deduce caller from formals
-    if ("cumul" %in% names(pf))
-      caller <- "freq"
-    else if ("transpose" %in% names(pf))
-      caller <- "descr"
-    else if ("chisq" %in% names(pf))
-      caller <- "ctable"
-    else if ("graph.col" %in% names(pf))
-      caller <- "dfSummary"
   } else {
     pf$flag_by <- FALSE
   }
@@ -174,6 +163,23 @@ check_args <- function(mc, dotArgs) {
         warning("one or more elements in the rows argument appears more than once")
       }
     }
+    
+    if (!is.null(pf$na.val)) {
+      if (!isTRUE(test_character(pf$na.val, any.missing = FALSE, len = 1))) {
+        errmsg %+=% "invalid na.val value; must be character of length 1"
+      }
+      
+      # If na.val is specified and one of two conditions is TRUE, we set it
+      # back to its default (NULL) value:
+      # - There are <NA> values in the vector/factor or
+      # - na.val is not in a factor's levels
+      if ((nas_found <- anyNA(pf$x)) ||
+          (is.factor(pf$x) && !pf$na.val %in% levels(pf$x))) {
+        if (nas_found && !isTRUE(st_options("freq.silent")))
+          message("NA values detected - na.val will be ignored")
+        pf$na.val <- NULL
+      }
+    }
   }
   
   # freq & ctable arguments ----------------------------------------------------
@@ -187,7 +193,7 @@ check_args <- function(mc, dotArgs) {
     if (!identical(pf$weights, NA)) {
       if (is.null(pf$weights)) {
         errmsg %+=% "weights vector not found"
-      } else if (caller_orig != "FUN" && 
+      } else if (isFALSE(pf$flag_by) && 
                  length(pf$weights) != nrow(as.data.frame(pf$x))) {
         errmsg %+=% "weights vector must have same length as 'x'"      
       }
@@ -259,6 +265,20 @@ check_args <- function(mc, dotArgs) {
         }
       }
     }
+    
+    if (!is.null(pf$na.val)) {
+      if (!isTRUE(test_character(pf$na.val, any.missing = FALSE, len = 1))) {
+        errmsg %+=% "invalid na.val value; must be character of length 1"
+      }
+      if (anyNA(pf$x) ||
+          (is.factor(pf$x) && !pf$na.val %in% levels(pf$x))) {
+          pf$na.val.x <- NULL
+      }
+      if (anyNA(pf$y) ||
+          (is.factor(pf$y) && !pf$na.val %in% levels(pf$y))) {
+        pf$na.val.y <- NULL
+      }
+    }
   }
   
   # descr arguments ------------------------------------------------------------
@@ -288,7 +308,7 @@ check_args <- function(mc, dotArgs) {
         }
       } else {
         # order has length > 1 -- all elements must correspond to column names
-        if (length(ind <- which(!pf$order %in% colnames(pf$x.df))) > 0) {
+        if (length(ind <- which(!pf$order %in% colnames(pf$xx))) > 0) {
           errmsg %+=% paste("Following ordering element(s) not recognized:", 
                             paste(pf$order[ind], sep = ", "),
                             collapse = " ")
@@ -296,10 +316,8 @@ check_args <- function(mc, dotArgs) {
       }
     }
     
-    if (!identical(pf$weights, NA)) {
-      if (is.null(pf$weights)) {
-        errmsg %+=% "weights vector not found"
-      } else if (caller_orig != "FUN" && (length(pf$weights) != nrow(pf$x.df))) {
+    if (!is.null(pf$weights)) {
+      if (isFALSE(pf$flag_by) && (length(pf$weights) != nrow(pf$xx))) {
         errmsg %+=% "weights vector must have same length as 'x'"      
       }
     }
@@ -386,17 +404,17 @@ check_args_tb <- function(mc) {
       !isTRUE(test_choice(pf$order, c(1, 2, 3)))) {
     errmsg %+=% "'order' must be one of 1, 2, or 3"
   }
-  
-  if ("na.rm" %in% names(mc) && 
-      !isTRUE(test_logical(pf$na.rm, len = 1, any.missing = FALSE))) {
-    errmsg %+=% "'na.rm' must be either TRUE or FALSE"
-  }
 
   if ("drop.val.col" %in% names(mc) && 
       !isTRUE(test_logical(pf$drop.val.col, len = 1, any.missing = FALSE))) {
     errmsg %+=% "'drop.val.col' must be either TRUE or FALSE"
   }
   
+  if ("na.rm" %in% names(mc)) {
+    message("parameter na.rm is deprecated; use ",
+            "freq(..., report.nas = FALSE) instead")
+  }
+  
   return(errmsg)
 }
 
@@ -449,14 +467,6 @@ check_args_print <- function(mc) {
      errmsg %+=% "'file' path is not valid - check that directory exists"
   }
   
-  # # Change method to browser when file name was (most likely) provided by user
-  # if (grepl("\\.html$", pf$file, ignore.case = TRUE, perl = TRUE) &&
-  #     !grepl(pattern = tempdir(), x = pf$file, fixed = TRUE) && 
-  #     pf$method == "pander") {
-  #   pf$method <- "browser"
-  #   message("Switching method to 'browser'")
-  # }
-  # 
   if (pf$method == "pander" && !is.na(pf$table.classes)) {
     errmsg %+=% "'table.classes' option does not apply to method 'pander'"
   }
@@ -560,6 +570,11 @@ check_args_st_options <- function(mc) {
                            len = 1, any.missing = FALSE))) {
     errmsg %+=% "'display.labels' must be either TRUE or FALSE"
   }
+
+  if ("na.val" %in% names(mc) && !is.null(pf$na.val) &&
+      isFALSE(test_character(pf$na.val, any.missing = FALSE, len = 1))) {
+        errmsg %+=% "invalid na.val; must be character vector of length 1"
+  }
   
   if ("bootstrap.css" %in% names(mc) &&
       !isTRUE(test_logical(pf$bootstrap.css, 
@@ -613,22 +628,23 @@ check_args_st_options <- function(mc) {
     errmsg %+=% "'ctable.totals' must be either TRUE or FALSE"
   }
   
-  if ("descr_stats" %in% names(mc)) {
-    valid_stats <- c("mean", "sd", "min", "q1", "med", "q3","max", "mad", 
-                     "iqr", "cv", "skewness", "se.skewness", "kurtosis", 
-                     "n.valid", "pct.valid")
+  if ("descr.stats" %in% names(mc)) {
     
-    if (length(pf$descr_stats) == 1 && 
-        !(pf$descr_stats %in% c("fivevnum", "common")) &&
-        !(pf$descr_stats %in% valid_stats)) {
+    # Check for invalid items
+    stats <- tolower(pf$descr.stats)
+    invalid_stats <- setdiff(
+      stats, c(.st_env$descr.stats.valid$no_wgts,
+               paste0("-", .st_env$descr.stats.valid$no_wgts),
+               "all", "common", "fivenum"))
+    
+    if (length(invalid_stats) > 0) {
       errmsg %+=%
-        paste("'descr_stats' value", dQuote(pf$descr_stats), "not recognized;",
-              "allowed values are: ", 
-              paste('"fivenum", "common", or a combination of :',
-                    paste0(dQuote(valid_stats), sep = ", ")))
+        paste("descr.stats: values", 
+              paste(dQuote(invalid_stats), collapse = ", "),
+              "not recognized; see ?descr")
     }
   }
-  
+
   if ("descr.transpose" %in% names(mc) &&
       !isTRUE(test_logical(pf$descr.transpose, len = 1, any.missing = FALSE))) {
     errmsg %+=% "'descr.transpose' must be either TRUE or FALSE"
