@@ -30,6 +30,9 @@
 #'   provide a custom string. Applies only to \emph{html} outputs.
 #' @param display.labels Logical. \code{TRUE} by default. Set to \code{FALSE} to
 #'   omit data frame and variable labels in the headings section.
+#' @param na.val Character. For factors and character vectors, consider this
+#'   value as \code{NA}. Ignored if there are actual NA values or if it matches
+#'   no value / factor level in the data. \code{NULL} by default.
 #' @param bootstrap.css Logical. Specifies whether to include 
 #'   \emph{Bootstrap css} in \emph{html} reports' \emph{head} section.
 #'   Defaults to \code{TRUE}. Set to \code{FALSE} when using the \dQuote{render}
@@ -59,6 +62,7 @@
 #' @param ctable.totals Logical. Corresponds to the \code{totals} parameter of
 #'   \code{\link{ctable}}. \code{TRUE} by default.
 #' @param ctable.round.digits Numeric. Defaults to \code{1}.
+#' @param ctable.silent Logical. Hide console messages. \code{FALSE} by default.
 #' @param descr.stats Character. Corresponds to the \code{stats} parameter of
 #'   \code{\link{descr}}. Defaults to \dQuote{all}.
 #' @param descr.transpose Logical. Corresponds to the \code{transpose} parameter
@@ -68,6 +72,9 @@
 #'   \dQuote{grid} for \emph{R Markdown} documents.
 #' @param dfSummary.varnumbers Logical. In \code{\link{dfSummary}}, display
 #'   variable numbers in the first column. Defaults to \code{TRUE}.
+#' @param dfSummary.class Logical. Show data classes in Name column.
+#'   \code{TRUE} by default.
+#'   variable numbers in the first column. Defaults to \code{TRUE}.
 #' @param dfSummary.labels.col Logical. In \code{\link{dfSummary}}, display
 #'   variable labels Defaults to \code{TRUE}.
 #' @param dfSummary.valid.col Logical. In \code{\link{dfSummary}}, include
@@ -190,6 +197,7 @@ st_options <- function(option                 = NULL,
                        headings               = TRUE,
                        footnote               = "default",
                        display.labels         = TRUE,
+                       na.val                 = NULL,
                        bootstrap.css          = TRUE,
                        custom.css             = NA_character_,
                        escape.pipe            = FALSE,
@@ -202,11 +210,13 @@ st_options <- function(option                 = NULL,
                        ctable.prop            = "r",
                        ctable.totals          = TRUE,
                        ctable.round.digits    = 1,
+                       ctable.silent          = FALSE,
                        descr.stats            = "all",
                        descr.transpose        = FALSE,
                        descr.silent           = FALSE,
                        dfSummary.style        = "multiline",
                        dfSummary.varnumbers   = TRUE,
+                       dfSummary.class        = TRUE,
                        dfSummary.labels.col   = TRUE,
                        dfSummary.valid.col    = TRUE,
                        dfSummary.na.col       = TRUE,
@@ -282,6 +292,7 @@ st_options <- function(option                 = NULL,
                    "headings"               = TRUE,
                    "footnote"               = "default",
                    "display.labels"         = TRUE,
+                   "na.val"                 = NULL,
                    "bootstrap.css"          = TRUE,
                    "custom.css"             = NA_character_,
                    "escape.pipe"            = FALSE,
@@ -294,11 +305,13 @@ st_options <- function(option                 = NULL,
                    "ctable.prop"            = "r",
                    "ctable.totals"          = TRUE,
                    "ctable.round.digits"    = 1,
+                   "ctable.silent"          = FALSE,
                    "descr.stats"            = "all",
                    "descr.transpose"        = FALSE,
                    "descr.silent"           = FALSE,
                    "dfSummary.style"        = "multiline",
                    "dfSummary.varnumbers"   = TRUE,
+                   "dfSummary.class"        = TRUE,
                    "dfSummary.labels.col"   = TRUE,
                    "dfSummary.graph.col"    = TRUE,
                    "dfSummary.valid.col"    = TRUE,
